@@ -73,14 +73,14 @@ static int wrap_saved_tempcount=0;
 
 #define WRAPPER_GROW	8
 
-void wrap_add_entry PROTO((WrapperEntry *e,int temp));
-void wrap_kill PROTO((void));
-void wrap_kill_temp PROTO((void));
-void wrap_free_entry PROTO((WrapperEntry *e));
-void wrap_free_entry_internal PROTO((WrapperEntry *e));
-void wrap_restore_saved PROTO((void));
+void wrap_add_entry (WrapperEntry *e,int temp);
+void wrap_kill (void);
+void wrap_kill_temp (void);
+void wrap_free_entry (WrapperEntry *e);
+void wrap_free_entry_internal (WrapperEntry *e);
+void wrap_restore_saved (void);
 
-void wrap_setup()
+void wrap_setup(void)
 {
     /* FIXME-reentrancy: if we do a multithreaded server, will need to
        move this to a per-connection data structure, or better yet
@@ -93,7 +93,9 @@ void wrap_setup()
     else
         wrap_setup_already_done = 1;
 
+#ifdef CLIENT_SUPPORT
     if (!current_parsed_root->isremote)
+#endif
     {
 	char *file;
 
@@ -154,7 +156,7 @@ void wrap_setup()
 /* Send -W arguments for the wrappers to the server.  The command must
    be one that accepts them (e.g. update, import).  */
 void
-wrap_send ()
+wrap_send (void)
 {
     int i;
 
@@ -196,9 +198,7 @@ wrap_send ()
  *
  * If first_call_p is nonzero, then start afresh.  */
 void
-wrap_unparse_rcs_options (line, first_call_p)
-    char **line;
-    int first_call_p;
+wrap_unparse_rcs_options (char **line, int first_call_p)
 {
     /* FIXME-reentrancy: we should design a reentrant interface, like
        a callback which gets handed each wrapper (a multithreaded
@@ -238,7 +238,7 @@ wrap_unparse_rcs_options (line, first_call_p)
  * Remove fmt str specifier other than %% or %s. And allow
  * only max_s %s specifiers
  */
-void
+static void
 wrap_clean_fmt_str(char *fmt, int max_s)
 {
     while (*fmt) {
@@ -264,9 +264,7 @@ wrap_clean_fmt_str(char *fmt, int max_s)
  * argument is set.
  */
 void
-wrap_add_file (file, temp)
-    const char *file;
-    int temp;
+wrap_add_file (const char *file, int temp)
 {
     FILE *fp;
     char *line = NULL;
@@ -276,7 +274,6 @@ wrap_add_file (file, temp)
     wrap_kill_temp ();
 
     /* Load the file.  */
-    errno = 0; /* Standard C doesn't require errno be set on error */
     fp = CVS_FOPEN (file, "r");
     if (fp == NULL)
     {
@@ -295,7 +292,7 @@ wrap_add_file (file, temp)
 }
 
 void
-wrap_kill()
+wrap_kill(void)
 {
     wrap_kill_temp();
     while(wrap_count)
@@ -303,7 +300,7 @@ wrap_kill()
 }
 
 void
-wrap_kill_temp()
+wrap_kill_temp(void)
 {
     WrapperEntry **temps=wrap_list+wrap_count;
 
@@ -312,16 +309,14 @@ wrap_kill_temp()
 }
 
 void
-wrap_free_entry(e)
-     WrapperEntry *e;
+wrap_free_entry(WrapperEntry *e)
 {
     wrap_free_entry_internal(e);
     free(e);
 }
 
 void
-wrap_free_entry_internal(e)
-    WrapperEntry *e;
+wrap_free_entry_internal(WrapperEntry *e)
 {
     free (e->wildCard);
     if (e->tocvsFilter)
@@ -333,7 +328,7 @@ wrap_free_entry_internal(e)
 }
 
 void
-wrap_restore_saved()
+wrap_restore_saved(void)
 {
     if(!wrap_saved_list)
 	return;
@@ -352,9 +347,7 @@ wrap_restore_saved()
 }
 
 void
-wrap_add (line, isTemp)
-   char *line;
-   int         isTemp;
+wrap_add (char *line, int isTemp)
 {
     char *temp;
     char ctemp;
@@ -417,8 +410,7 @@ wrap_add (line, isTemp)
 	switch(opt){
 	case 'f':
 	    /* Before this is reenabled, need to address the problem in
-	       commit.c (see
-	       http://ximbiot.com/cvs/cvshome/docs/infowrapper.html).  */
+	       commit.c (see http://www.cvshome.org/docs/infowrapper.html).  */
 	    error (1, 0,
 		   "-t/-f wrappers not supported by this version of CVS");
 
@@ -426,14 +418,13 @@ wrap_add (line, isTemp)
 		free(e.fromcvsFilter);
 	    /* FIXME: error message should say where the bad value
 	       came from.  */
-	    e.fromcvsFilter=expand_path (temp, "<wrapper>", 0);
+	    e.fromcvsFilter=expand_path (temp, "<wrapper>", 0, 0);
             if (!e.fromcvsFilter)
 		error (1, 0, "Correct above errors first");
 	    break;
 	case 't':
 	    /* Before this is reenabled, need to address the problem in
-	       commit.c (see
-	       http://ximbiot.com/cvs/cvshome/docs/infowrapper.html).  */
+	       commit.c (see http://www.cvshome.org/docs/infowrapper.html).  */
 	    error (1, 0,
 		   "-t/-f wrappers not supported by this version of CVS");
 
@@ -441,7 +432,7 @@ wrap_add (line, isTemp)
 		free(e.tocvsFilter);
 	    /* FIXME: error message should say where the bad value
 	       came from.  */
-	    e.tocvsFilter=expand_path (temp, "<wrapper>", 0);
+	    e.tocvsFilter=expand_path (temp, "<wrapper>", 0, 0);
             if (!e.tocvsFilter)
 		error (1, 0, "Correct above errors first");
 	    break;
@@ -468,9 +459,7 @@ wrap_add (line, isTemp)
 }
 
 void
-wrap_add_entry(e, temp)
-    WrapperEntry *e;
-    int temp;
+wrap_add_entry(WrapperEntry *e, int temp)
 {
     int x;
     if(wrap_count+wrap_tempcount>=wrap_size){
@@ -492,9 +481,7 @@ wrap_add_entry(e, temp)
 
 /* Return 1 if the given filename is a wrapper filename */
 int
-wrap_name_has (name,has)
-    const char   *name;
-    WrapMergeHas  has;
+wrap_name_has (const char *name, WrapMergeHas has)
 {
     int x,count=wrap_count+wrap_tempcount;
     char *temp;
@@ -522,11 +509,10 @@ wrap_name_has (name,has)
     return (0);
 }
 
-static WrapperEntry *wrap_matching_entry PROTO ((const char *));
+static WrapperEntry *wrap_matching_entry (const char *);
 
 static WrapperEntry *
-wrap_matching_entry (name)
-    const char *name;
+wrap_matching_entry (const char *name)
 {
     int x,count=wrap_count+wrap_tempcount;
 
@@ -540,9 +526,7 @@ wrap_matching_entry (name)
    ASFLAG, then include "-k" at the beginning (e.g. "-kb"), otherwise
    just give the option itself (e.g. "b").  */
 char *
-wrap_rcsoption (filename, asflag)
-    const char *filename;
-    int asflag;
+wrap_rcsoption (const char *filename, int asflag)
 {
     WrapperEntry *e = wrap_matching_entry (filename);
     char *buf;
@@ -564,8 +548,7 @@ wrap_rcsoption (filename, asflag)
 }
 
 char *
-wrap_tocvs_process_file(fileName)
-    const char *fileName;
+wrap_tocvs_process_file(const char *fileName)
 {
     WrapperEntry *e=wrap_matching_entry(fileName);
     static char *buf = NULL;
@@ -592,8 +575,7 @@ wrap_tocvs_process_file(fileName)
 }
 
 int
-wrap_merge_is_copy (fileName)
-    const char *fileName;
+wrap_merge_is_copy (const char *fileName)
 {
     WrapperEntry *e=wrap_matching_entry(fileName);
     if(e==NULL || e->mergeMethod==WRAP_MERGE)
@@ -603,8 +585,7 @@ wrap_merge_is_copy (fileName)
 }
 
 void
-wrap_fromcvs_process_file(fileName)
-    const char *fileName;
+wrap_fromcvs_process_file(const char *fileName)
 {
     char *args;
     WrapperEntry *e=wrap_matching_entry(fileName);
