@@ -19,7 +19,7 @@
 #include "cvs.h"
 #include <assert.h>
 
-static int deep_remove_dir PROTO((const char *path));
+static int deep_remove_dir( const char *path );
 
 /*
  * Copies "from" to "to".
@@ -655,7 +655,7 @@ xcmp (file1_file, file2_file)
     (void) close (fd2);
     return (ret);
 }
-
+
 unsigned char
 VMS_filename_classes[] =
 {
@@ -693,8 +693,6 @@ VMS_filename_classes[] =
     0xf8,0xf9,0xfa,0xfb, 0xfc,0xfd,0xfe,0xff,
 };
 
-
-
 /* Like strcmp, but with the appropriate tweaks for file names.
    Under VMS, filenames are case-insensitive but case-preserving.
    FIXME: this should compare y.tab.c equal with y_tab.c, at least
@@ -939,7 +937,7 @@ xresolvepath ( path )
 }
 
 /* Return a pointer into PATH's last component.  */
-const char *
+char *
 last_component (path)
     char *path;
 {
